from __future__ import annotations

import mteb
import mteb.task_aggregation as task_aggregation

# define some test data
bitext1_1 = mteb.MTEBResults(
    dataset_revision="test_rev",
    task_name="BornholmBitextMining",
    mteb_version="test_version",
    evaluation_time=1,
    scores={"test": [{"main_score": 1, "hf_subset": "NaN", "languages": ["eng-Latn"]}]},
)

bitext1_2 = mteb.MTEBResults(
    dataset_revision="test_rev",
    task_name="BornholmBitextMining",
    mteb_version="test_version",
    evaluation_time=1,
    scores={"test": [{"main_score": 2, "hf_subset": "NaN", "languages": ["eng-Latn"]}]},
)

classification1_1 = mteb.MTEBResults(
    dataset_revision="test_rev",
    task_name="Banking77Classification",
    mteb_version="test_version",
    evaluation_time=1,
    scores={"test": [{"main_score": 1, "hf_subset": "NaN", "languages": ["eng-Latn"]}]},
)

classification1_2 = mteb.MTEBResults(
    dataset_revision="test_rev",
    task_name="Banking77Classification",
    mteb_version="test_version",
    evaluation_time=1,
    scores={"test": [{"main_score": 2, "hf_subset": "NaN", "languages": ["eng-Latn"]}]},
)

classification2_1 = mteb.MTEBResults(
    dataset_revision="test_rev",
    task_name="AfriSentiClassification",
    mteb_version="test_version",
    evaluation_time=1,
    scores={"test": [{"main_score": 1, "hf_subset": "NaN", "languages": ["eng-Latn"]}]},
)

mteb_results = {
    "model1": {
        "rev1": [bitext1_1, classification1_2, classification2_1],
        "rev2": [bitext1_1, classification1_1, classification2_1],
    },
    "model2": {
        "rev1": [bitext1_2, classification1_2, classification2_1],
        "rev2": [bitext1_2, classification1_1, classification2_1],
    },
}


def test_mean():
    expected = {
        "model1": {
            "rev1": {"mean": 4 / 3},  # (1 + 2 + 1) / 3
            "rev2": {"mean": 1},  # (1 + 1 + 1) / 3
        },
        "model2": {
            "rev1": {"mean": 5 / 3},  # (2 + 2 + 1) / 3
            "rev2": {"mean": 4 / 3},  # (2 + 1 + 1) / 3
        },
    }

    assert task_aggregation.mean(mteb_results) == expected


def test_task_category_weighted_mean():
    expected = {
        "model1": {
            "rev1": {
                "mean (BitextMining)": 1.0,
                "mean (Classification)": 1.5,
                "mean (weighted by task type)": 1.25,  # ( 1/1 + (2 + 1) / 2 ) / 2
            },
            "rev2": {
                "mean (BitextMining)": 1.0,
                "mean (Classification)": 1.0,
                "mean (weighted by task type)": 1.0,  # ( 1/1 + (1 + 1) / 2 ) / 2
            },
        },
        "model2": {
            "rev1": {
                "mean (BitextMining)": 2.0,
                "mean (Classification)": 1.5,
                "mean (weighted by task type)": 1.75,  # ( 2/1 + (2 + 1) / 2 ) / 2
            },
            "rev2": {
                "mean (BitextMining)": 2.0,
                "mean (Classification)": 1.0,
                "mean (weighted by task type)": 1.5,  # ( 2/1 + (1 + 1) / 2 ) / 2
            },
        },
    }

    assert task_aggregation.task_category_weighted_mean(mteb_results) == expected


def test_borda_count_simple():
    mteb_results_simple = {
        "model1": {
            "rev1": [bitext1_1],
        },
        "model2": {
            "rev2": [bitext1_2],
        },
    }
    expected = {
        "model1": {
            "rev1": {"borda_count": 0},
        },
        "model2": {
            "rev2": {"borda_count": 1},
        },
    }
    assert task_aggregation.borda_count(mteb_results_simple) == expected


def test_borda_count_simple_with_tie():
    mteb_results_simple_with_tie = {
        "model1": {
            "rev1": [bitext1_1],
            "rev2": [bitext1_1],
        },
        "model2": {
            "rev1": [bitext1_2],
            "rev2": [bitext1_2],
        },
    }
    expected = {
        "model1": {
            "rev1": {"borda_count": 0.5},
            "rev2": {"borda_count": 0.5},
        },
        "model2": {
            "rev1": {"borda_count": 2.5},
            "rev2": {"borda_count": 2.5},
        },
    }
    assert task_aggregation.borda_count(mteb_results_simple_with_tie) == expected


def test_borda_count_multiple_task_and_ties():
    # task 1: model1/rev1 == model2/rev1 > model1/rev2 == model2/rev2
    # task 2: model1/rev1 == model2/rev1 > model1/rev2 == model2/rev2
    # task 3: model1/rev1 == model2/rev1 == model1/rev2 == model2/rev2
    # given there is 4 candidates the max score is 3 (4 - 1)
    # we use tournament borda count so shared ranks get the average of the ranks they would have gotten

    expected = {
        "model1": {
            "rev1": {"borda_count": 0.5 + 2.5 + (6 / 4)},
            "rev2": {"borda_count": 0.5 + 0.5 + (6 / 4)},
        },
        "model2": {
            "rev1": {"borda_count": 2.5 + 2.5 + (6 / 4)},
            "rev2": {"borda_count": 2.5 + 0.5 + (6 / 4)},
        },
    }

    assert task_aggregation.borda_count(mteb_results) == expected



from __future__ import annotations

import datasets

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class SadeemQuestionRetrieval(AbsTaskRetrieval):
    _EVAL_SPLIT = "test"

    metadata = TaskMetadata(
        name="SadeemQuestionRetrieval",
        dataset={
            "path": "sadeem-ai/sadeem-ar-eval-retrieval-questions",
            "revision": "3cb0752b182e5d5d740df547748b06663c8e0bd9",
            "name": "test",
        },
        reference="https://huggingface.co/datasets/sadeem-ai/sadeem-ar-eval-retrieval-questions",
        description="SadeemQuestion: A Benchmark Data Set for Community Question-Retrieval Research",
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=[_EVAL_SPLIT],
        eval_langs=["ara-Arab"],
        main_score="ndcg_at_10",
        date=("2024-01-01", "2024-04-01"),
        domains=["Written", "Written"],
        task_subtypes=["Article retrieval"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""
            @inproceedings{sadeem-2024-ar-retrieval-questions,
            title = "SadeemQuestionRetrieval: A New Benchmark for Arabic questions-based Articles Searching.",
            author = "abubakr.soliman@sadeem.app"
            }
        """,
        descriptive_stats={
            "n_samples": {_EVAL_SPLIT: 22979},
            "avg_character_length": {_EVAL_SPLIT: 500.0},
        },
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        query_list = datasets.load_dataset(**self.metadata_dict["dataset"])["queries"]
        queries = {row["query-id"]: row["text"] for row in query_list}

        corpus_list = datasets.load_dataset(**self.metadata_dict["dataset"])["corpus"]
        corpus = {row["corpus-id"]: {"text": row["text"]} for row in corpus_list}

        qrels_list = datasets.load_dataset(**self.metadata_dict["dataset"])["qrels"]
        qrels = {row["query-id"]: {row["corpus-id"]: 1} for row in qrels_list}

        self.corpus = {self._EVAL_SPLIT: corpus}
        self.queries = {self._EVAL_SPLIT: queries}
        self.relevant_docs = {self._EVAL_SPLIT: qrels}

        self.data_loaded = True



from __future__ import annotations

from mteb.abstasks.AbsTaskClassification import AbsTaskClassification
from mteb.abstasks.TaskMetadata import TaskMetadata


class FinancialPhrasebankClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="FinancialPhrasebankClassification",
        description="Polar sentiment dataset of sentences from financial news, categorized by sentiment into positive, negative, or neutral.",
        reference="https://arxiv.org/abs/1307.5336",
        dataset={
            "path": "takala/financial_phrasebank",
            "revision": "1484d06fe7af23030c7c977b12556108d1f67039",
            "name": "sentences_allagree",
            "trust_remote_code": True,
        },
        type="Classification",
        category="s2s",
        modalities=["text"],
        eval_splits=["train"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2013-11-01", "2013-11-01"),
        domains=["News", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-nc-sa-3.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""
            @article{Malo2014GoodDO,
            title={Good debt or bad debt: Detecting semantic orientations in economic texts},
            author={P. Malo and A. Sinha and P. Korhonen and J. Wallenius and P. Takala},
            journal={Journal of the Association for Information Science and Technology},
            year={2014},
            volume={65}
            }
        """,
        descriptive_stats={
            "n_samples": {"train": 4840},
            "avg_character_length": {"train": 121.96},
        },
    )

    def dataset_transform(self):
        self.dataset = self.dataset.rename_column("sentence", "text")



from __future__ import annotations

from mteb.abstasks.MultilingualTask import MultilingualTask
from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskReranking import AbsTaskReranking

_EVAL_LANGS = {
    "bg": ["bul-Cyrl"],
    "bn": ["ben-Beng"],
    "cs": ["ces-Latn"],
    "da": ["dan-Latn"],
    "de": ["deu-Latn"],
    "en": ["eng-Latn"],
    "fa": ["fas-Arab"],
    "fi": ["fin-Latn"],
    "hi": ["hin-Deva"],
    "it": ["ita-Latn"],
    "nl": ["nld-Latn"],
    "pt": ["por-Latn"],
    "ro": ["ron-Latn"],
    "sr": ["srp-Cyrl"],
    "no": ["nor-Latn"],
    "sv": ["swe-Latn"],
}


class WikipediaRerankingMultilingual(MultilingualTask, AbsTaskReranking):
    metadata = TaskMetadata(
        name="WikipediaRerankingMultilingual",
        description="The dataset is derived from Cohere's wikipedia-2023-11 dataset and contains synthetically generated queries.",
        reference="https://huggingface.co/datasets/ellamind/wikipedia-2023-11-reranking-multilingual",
        dataset={
            "path": "ellamind/wikipedia-2023-11-reranking-multilingual",
            "revision": "6268b37d6f975f2a134791ba2f250a91d0bdfb4f",
        },
        type="Reranking",
        category="s2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=_EVAL_LANGS,
        main_score="map",
        date=("2023-11-01", "2024-05-15"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=[],
        license="cc-by-sa-3.0",
        annotations_creators="LM-generated and reviewed",
        dialect=[],
        sample_creation="LM-generated and verified",
        bibtex_citation="""@ONLINE{wikidump,
    author = "Wikimedia Foundation",
    title  = "Wikimedia Downloads",
    url    = "https://dumps.wikimedia.org"
}""",
        descriptive_stats={
            "n_samples": {
                "en": 1500,
                "de": 1500,
                "it": 1500,
                "pt": 1500,
                "nl": 1500,
                "cs": 1500,
                "ro": 1500,
                "bg": 1500,
                "sr": 1500,
                "fi": 1500,
                "da": 1500,
                "fa": 1500,
                "hi": 1500,
                "bn": 1500,
                "no": 1500,
                "sv": 1500,
            },
            "test": {
                "num_samples": 24000,
                "num_positive": 24000,
                "num_negative": 24000,
                "avg_query_len": 59.091208333333334,
                "avg_positive_len": 1.0,
                "avg_negative_len": 8.0,
                "hf_subset_descriptive_stats": {
                    "bg": {
                        "num_samples": 1500,
                        "num_positive": 1500,
                        "num_negative": 1500,
                        "avg_query_len": 60.82666666666667,
                        "avg_positive_len": 1.0,
                        "avg_negative_len": 8.0,
                    },
                    "bn": {
                        "num_samples": 1500,
                        "num_positive": 1500,
                        "num_negative": 1500,
                        "avg_query_len": 47.266666666666666,
                        "avg_positive_len": 1.0,
                        "avg_negative_len": 8.0,
                    },
                    "cs": {
                        "num_samples": 1500,
                        "num_positive": 1500,
                        "num_negative": 1500,
                        "avg_query_len": 56.272,
                        "avg_positive_len": 1.0,
                        "avg_negative_len": 8.0,
                    },
                    "da": {
                        "num_samples": 1500,
                        "num_positive": 1500,
                        "num_negative": 1500,
                        "avg_query_len": 56.75066666666667,
                        "avg_positive_len": 1.0,
                        "avg_negative_len": 8.0,
                    },
                    "de": {
                        "num_samples": 1500,
                        "num_positive": 1500,
                        "num_negative": 1500,
                        "avg_query_len": 70.004,
                        "avg_positive_len": 1.0,
                        "avg_negative_len": 8.0,
                    },
                    "en": {
                        "num_samples": 1500,
                        "num_positive": 1500,
                        "num_negative": 1500,
                        "avg_query_len": 68.372,
                        "avg_positive_len": 1.0,
                        "avg_negative_len": 8.0,
                    },
                    "fa": {
                        "num_samples": 1500,
                        "num_positive": 1500,
                        "num_negative": 1500,
                        "avg_query_len": 48.66733333333333,
                        "avg_positive_len": 1.0,
                        "avg_negative_len": 8.0,
                    },
                    "fi": {
                        "num_samples": 1500,
                        "num_positive": 1500,
                        "num_negative": 1500,
                        "avg_query_len": 55.343333333333334,
                        "avg_positive_len": 1.0,
                        "avg_negative_len": 8.0,
                    },
                    "hi": {
                        "num_samples": 1500,
                        "num_positive": 1500,
                        "num_negative": 1500,
                        "avg_query_len": 50.77733333333333,
                        "avg_positive_len": 1.0,
                        "avg_negative_len": 8.0,
                    },
                    "it": {
                        "num_samples": 1500,
                        "num_positive": 1500,
                        "num_negative": 1500,
                        "avg_query_len": 70.05466666666666,
                        "avg_positive_len": 1.0,
                        "avg_negative_len": 8.0,
                    },
                    "nl": {
                        "num_samples": 1500,
                        "num_positive": 1500,
                        "num_negative": 1500,
                        "avg_query_len": 65.34466666666667,
                        "avg_positive_len": 1.0,
                        "avg_negative_len": 8.0,
                    },
                    "pt": {
                        "num_samples": 1500,
                        "num_positive": 1500,
                        "num_negative": 1500,
                        "avg_query_len": 65.11933333333333,
                        "avg_positive_len": 1.0,
                        "avg_negative_len": 8.0,
                    },
                    "ro": {
                        "num_samples": 1500,
                        "num_positive": 1500,
                        "num_negative": 1500,
                        "avg_query_len": 61.973333333333336,
                        "avg_positive_len": 1.0,
                        "avg_negative_len": 8.0,
                    },
                    "sr": {
                        "num_samples": 1500,
                        "num_positive": 1500,
                        "num_negative": 1500,
                        "avg_query_len": 55.669333333333334,
                        "avg_positive_len": 1.0,
                        "avg_negative_len": 8.0,
                    },
                    "no": {
                        "num_samples": 1500,
                        "num_positive": 1500,
                        "num_negative": 1500,
                        "avg_query_len": 55.288,
                        "avg_positive_len": 1.0,
                        "avg_negative_len": 8.0,
                    },
                    "sv": {
                        "num_samples": 1500,
                        "num_positive": 1500,
                        "num_negative": 1500,
                        "avg_query_len": 57.73,
                        "avg_positive_len": 1.0,
                        "avg_negative_len": 8.0,
                    },
                },
            },
        },
    )



from __future__ import annotations

from mteb.abstasks.AbsTaskSpeedTask import AbsTaskSpeedTask
from mteb.abstasks.TaskMetadata import TaskMetadata


class CPUSpeedTask(AbsTaskSpeedTask):
    metadata = TaskMetadata(
        name="CPUSpeedTask",
        description="Time taken to encode the text 'The Ugly Duckling' split by paragraphs on a CPU.",
        reference="https://github.com/KennethEnevoldsen/scandinavian-embedding-benchmark/blob/c8376f967d1294419be1d3eb41217d04cd3a65d3/src/seb/registered_tasks/speed.py#L83-L96",
        dataset={"path": " ", "revision": "1.0"},
        type="Speed",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="avg_words_per_sec",
        date=("2024-06-20", "2024-06-20"),
        domains=["Fiction", "Written"],
        task_subtypes=[],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="",
        descriptive_stats={
            "n_samples": {"test": 1},
            "avg_character_length": {"test": 3591},
        },
    )



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskClustering import AbsTaskClustering


class WikiCitiesClustering(AbsTaskClustering):
    metadata = TaskMetadata(
        name="WikiCitiesClustering",
        description="Clustering of Wikipedia articles of cities by country from https://huggingface.co/datasets/wikipedia. Test set includes 126 countries, and a total of 3531 cities.",
        reference="https://huggingface.co/datasets/wikipedia",
        dataset={
            "path": "jinaai/cities_wiki_clustering",
            "revision": "ddc9ee9242fa65332597f70e967ecc38b9d734fa",
        },
        type="Clustering",
        category="p2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="v_measure",
        date=("2000-01-01", "2021-12-31"),  # very rough estimate
        domains=["Encyclopaedic", "Written"],
        task_subtypes=[],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@ONLINE{wikidump,
    author = "Wikimedia Foundation",
    title  = "Wikimedia Downloads",
    url    = "https://dumps.wikimedia.org"
}""",
        descriptive_stats={"n_samples": None, "avg_character_length": None},
    )



"""test mteb.MTEB's integration with SentenceTransformers"""

from __future__ import annotations

import logging

import pytest

import mteb
from mteb import MTEB
from mteb.abstasks import AbsTask

from .mock_models import MockNumpyEncoder
from .task_grid import TASK_TEST_GRID

logging.basicConfig(level=logging.INFO)


@pytest.mark.parametrize("task", TASK_TEST_GRID)
@pytest.mark.parametrize("model", [MockNumpyEncoder()])
def test_benchmark_sentence_transformer(task: str | AbsTask, model: mteb.Encoder):
    """Test that a task can be fetched and run"""
    eval = MTEB(tasks=[task])
    eval.run(model, output_folder="tests/results", overwrite_results=True)



# SuperLIM tasks
from __future__ import annotations

from mteb.abstasks.AbsTaskClassification import AbsTaskClassification
from mteb.abstasks.TaskMetadata import TaskMetadata


class DalajClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="DalajClassification",
        dataset={
            "path": "AI-Sweden/SuperLim",
            "revision": "7ebf0b4caa7b2ae39698a889de782c09e6f5ee56",
            "name": "dalaj",
            "trust_remote_code": True,
        },
        description="A Swedish dataset for linguistic acceptability. Available as a part of Superlim.",
        reference="https://spraakbanken.gu.se/en/resources/superlim",
        type="Classification",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["swe-Latn"],
        main_score="accuracy",
        date=("2017-01-01", "2020-12-31"),
        domains=["Non-fiction", "Written"],
        task_subtypes=["Linguistic acceptability"],
        license="cc-by-4.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="created",
        bibtex_citation="""@misc{2105.06681,
Author = {Elena Volodina and Yousuf Ali Mohammed and Julia Klezl},
Title = {DaLAJ - a dataset for linguistic acceptability judgments for Swedish: Format, baseline, sharing},
Year = {2021},
Eprint = {arXiv:2105.06681},
}""",
        descriptive_stats={
            "n_samples": {"test": 444},
            "avg_character_length": {"test": 243.8},
        },
    )

    @property
    def metadata_dict(self) -> dict[str, str]:
        metadata_dict = super().metadata_dict
        metadata_dict["n_experiments"] = 10
        metadata_dict["samples_per_label"] = 16
        return metadata_dict

    def dataset_transform(self):
        """This dataset consist of two columns of relevance, "original_sentence" and "corrected_sentence".
        We will use the original sentence as we "wrong" sentence and the corrected sentence as the "correct" sentence
        """

        def __convert_sample_to_classification(sample):
            text = sample["original_sentence"] + sample["corrected_sentence"]
            label = [1] * len(sample["original_sentence"]) + [0] * len(
                sample["corrected_sentence"]
            )
            return {"text": text, "label": label}

        columns_to_keep = ["original_sentence", "corrected_sentence"]
        for split in self.dataset:
            columns_names = self.dataset[split].column_names  # type: ignore
            columns_to_remove = [
                col for col in columns_names if col not in columns_to_keep
            ]
            self.dataset[split] = self.dataset[split].remove_columns(columns_to_remove)  # type: ignore

        self.dataset = self.dataset.map(
            __convert_sample_to_classification,
            batched=True,
            remove_columns=columns_to_keep,
        )



from __future__ import annotations

import os

from datasets import load_dataset
from huggingface_hub import create_repo, upload_file

_LANGUAGES = {
    "de": "German",
    "en": "English",
    "es": "Spanish",
    "fr": "French",
    "ja": "Japanese",
    "zh": "Chinese",
}
repo_name = "amazon_reviews_multi"
create_repo(repo_name, organization="mteb", repo_type="dataset")

for lang in _LANGUAGES:
    raw_dset = load_dataset("amazon_reviews_multi", lang)
    raw_dset = raw_dset.map(
        lambda x: {"text": x["review_title"] + "\n\n" + x["review_body"]}, num_proc=4
    )
    raw_dset = raw_dset.map(lambda x: {"label": x["stars"] - 1}, num_proc=4)
    raw_dset = raw_dset.map(lambda x: {"label_text": str(x["label"])}, num_proc=4)

    raw_dset = raw_dset.rename_column("review_id", "id")
    raw_dset = raw_dset.remove_columns(
        [
            "product_id",
            "reviewer_id",
            "review_body",
            "review_title",
            "language",
            "product_category",
            "stars",
        ]
    )

    for split, dset in raw_dset.items():
        save_path = f"{lang}/{split}.jsonl"
        dset.to_json(save_path)
        upload_file(
            path_or_fileobj=save_path,
            path_in_repo=save_path,
            repo_id="mteb/" + repo_name,
            repo_type="dataset",
        )
    os.system(f"rm -r {lang}")



from __future__ import annotations

from mteb.abstasks.AbsTaskClassification import AbsTaskClassification
from mteb.abstasks.TaskMetadata import TaskMetadata


class MarathiNewsClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="MarathiNewsClassification",
        description="A Marathi dataset for 3-class classification of Marathi news articles",
        reference="https://github.com/goru001/nlp-for-marathi",
        dataset={
            "path": "mlexplorer008/marathi_news_classification",
            "revision": "7640cf8132cca1f99995ac71512a670e3c965cf1",
        },
        type="Classification",
        category="s2s",
        modalities=["text"],
        date=("2014-01-01", "2018-01-01"),
        eval_splits=["test"],
        eval_langs=["mar-Deva"],
        main_score="f1",
        domains=["News", "Written"],
        task_subtypes=["Topic classification"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@article{kunchukuttan2020indicnlpcorpus,
    title={AI4Bharat-IndicNLP Corpus: Monolingual Corpora and Word Embeddings for Indic Languages},
    author={Anoop Kunchukuttan and Divyanshu Kakwani and Satish Golla and Gokul N.C. and Avik Bhattacharyya and Mitesh M. Khapra and Pratyush Kumar},
    year={2020},
    journal={arXiv preprint arXiv:2005.00085},
}""",
        descriptive_stats={
            "n_samples": {"test": 2048},
            "avg_character_length": {"test": 52.37},
        },
    )

    def dataset_transform(self):
        self.dataset = self.dataset.rename_columns({"headline": "text"})
        self.dataset = self.stratified_subsampling(self.dataset, seed=self.seed)



from __future__ import annotations

from mteb.abstasks.AbsTaskBitextMining import AbsTaskBitextMining
from mteb.abstasks.TaskMetadata import TaskMetadata


class BornholmBitextMining(AbsTaskBitextMining):
    metadata = TaskMetadata(
        name="BornholmBitextMining",
        dataset={
            "path": "strombergnlp/bornholmsk_parallel",
            "revision": "3bc5cfb4ec514264fe2db5615fac9016f7251552",
            "trust_remote_code": True,
        },
        description="Danish Bornholmsk Parallel Corpus. Bornholmsk is a Danish dialect spoken on the island of Bornholm, Denmark. Historically it is a part of east Danish which was also spoken in Scania and Halland, Sweden.",
        reference="https://aclanthology.org/W19-6138/",
        type="BitextMining",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["dan-Latn"],
        main_score="f1",
        date=("2019-01-01", "2019-12-31"),
        domains=["Web", "Social", "Fiction", "Written"],
        license="cc-by-4.0",
        task_subtypes=["Dialect pairing"],
        annotations_creators="expert-annotated",
        dialect=["da-dan-bornholm"],
        sample_creation="created",
        bibtex_citation="""
@inproceedings{derczynskiBornholmskNaturalLanguage2019,
	title = {Bornholmsk natural language processing: Resources and tools},
	url = {https://pure.itu.dk/ws/files/84551091/W19_6138.pdf},
	shorttitle = {Bornholmsk natural language processing},
	pages = {338--344},
	booktitle = {Proceedings of the Nordic Conference of Computational Linguistics (2019)},
	publisher = {Linköping University Electronic Press},
	author = {Derczynski, Leon and Kjeldsen, Alex Speed},
	urldate = {2024-04-24},
	date = {2019},
	file = {Available Version (via Google Scholar):/Users/au554730/Zotero/storage/FBQ73ZYN/Derczynski and Kjeldsen - 2019 - Bornholmsk natural language processing Resources .pdf:application/pdf},
}
""",
        descriptive_stats={
            "n_samples": {"test": 500},
            "test": {
                "average_sentence1_length": 49.834,
                "average_sentence2_length": 38.888,
                "num_samples": 500,
            },
        },
    )

    def dataset_transform(self):
        # Convert to standard format
        self.dataset = self.dataset.rename_column("da", "sentence1")
        self.dataset = self.dataset.rename_column("da_bornholm", "sentence2")



from __future__ import annotations

import logging
from typing import Any

from datasets import Dataset

from mteb.abstasks.MultilingualTask import MultilingualTask
from mteb.abstasks.TaskMetadata import TaskMetadata
from mteb.encoder_interface import Encoder, EncoderWithQueryCorpusEncode
from mteb.evaluation.evaluators import RerankingEvaluator
from mteb.load_results.mteb_results import ScoresDict

from ....abstasks.AbsTaskReranking import AbsTaskReranking

logger = logging.getLogger(__name__)

_EVAL_SPLIT = "dev"
_LANGUAGES = {
    "ar": ["ara-Arab"],
    "bn": ["ben-Beng"],
    "de": ["deu-Latn"],
    "en": ["eng-Latn"],
    "es": ["spa-Latn"],
    "fa": ["fas-Arab"],
    "fi": ["fin-Latn"],
    "fr": ["fra-Latn"],
    "hi": ["hin-Deva"],
    "id": ["ind-Latn"],
    "ja": ["jpn-Jpan"],
    "ko": ["kor-Kore"],
    "ru": ["rus-Cyrl"],
    "sw": ["swa-Latn"],
    "te": ["tel-Telu"],
    "th": ["tha-Thai"],
    "yo": ["yor-Latn"],
    "zh": ["zho-Hans"],
}

_CITATION = """@article{10.1162/tacl_a_00595,
    author = {Zhang, Xinyu and Thakur, Nandan and Ogundepo, Odunayo and Kamalloo, Ehsan and Alfonso-Hermelo, David and Li, Xiaoguang and Liu, Qun and Rezagholizadeh, Mehdi and Lin, Jimmy},
    title = "{MIRACL: A Multilingual Retrieval Dataset Covering 18 Diverse Languages}",
    journal = {Transactions of the Association for Computational Linguistics},
    volume = {11},
    pages = {1114-1131},
    year = {2023},
    month = {09},
    issn = {2307-387X},
    doi = {10.1162/tacl_a_00595},
}"""


class MIRACLReranking(MultilingualTask, AbsTaskReranking):
    metadata = TaskMetadata(
        name="MIRACLReranking",
        description="MIRACL (Multilingual Information Retrieval Across a Continuum of Languages) is a multilingual retrieval dataset that focuses on search across 18 different languages.",
        reference="https://project-miracl.github.io/",
        dataset={
            "path": "miracl/mmteb-miracl-reranking",
            "revision": "6d1962c527217f8927fca80f890f14f36b2802af",
            "trust_remote_code": True,
        },
        type="Reranking",
        category="s2s",
        modalities=["text"],
        eval_splits=[_EVAL_SPLIT],
        eval_langs=_LANGUAGES,
        main_score="NDCG@10(MIRACL)",
        date=("2022-06-01", "2023-01-30"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=[],
        license="cc-by-sa-4.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="created",
        bibtex_citation=_CITATION,
        descriptive_stats={
            "n_samples": {"dev": 44608},
            "avg_character_length": {"dev": 506.30},
        },
    )

    def _evaluate_subset(
        self,
        model: Encoder | EncoderWithQueryCorpusEncode,
        data_split: Dataset,
        *,
        encode_kwargs: dict[str, Any] = {},
        **kwargs: Any,
    ) -> ScoresDict:
        evaluator = RerankingEvaluator(
            samples=data_split,
            evaluator_type="miracl",
            task_name=self.metadata.name,
            encode_kwargs=encode_kwargs,
            **kwargs,
        )
        scores = evaluator(model)

        self._add_main_score(scores)
        return scores



from __future__ import annotations

from datasets import load_dataset

from mteb.abstasks.MultilingualTask import MultilingualTask
from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval

_EVAL_LANGS = {
    "bg": ["bul-Cyrl"],
    "bn": ["ben-Beng"],
    "cs": ["ces-Latn"],
    "da": ["dan-Latn"],
    "de": ["deu-Latn"],
    "en": ["eng-Latn"],
    "fa": ["fas-Arab"],
    "fi": ["fin-Latn"],
    "hi": ["hin-Deva"],
    "it": ["ita-Latn"],
    "nl": ["nld-Latn"],
    "pt": ["por-Latn"],
    "ro": ["ron-Latn"],
    "sr": ["srp-Cyrl"],
    "no": ["nor-Latn"],
    "sv": ["swe-Latn"],
}


# adapted from MIRACLRetrieval
def _load_data(
    path: str,
    langs: list,
    split: str,
    cache_dir: str = None,
    revision_queries: str = None,
    revision_corpus: str = None,
    revision_qrels: str = None,
):
    queries = {lang: {split: {}} for lang in langs}
    corpus = {lang: {split: {}} for lang in langs}
    qrels = {lang: {split: {}} for lang in langs}

    for lang in langs:
        queries_path = path
        corpus_path = path.replace("queries", "corpus")
        qrels_path = path.replace("queries", "qrels")
        queries_lang = load_dataset(
            queries_path,
            lang,
            split=split,
            cache_dir=cache_dir,
            revision=revision_queries,
        )
        corpus_lang = load_dataset(
            corpus_path,
            lang,
            split=split,
            cache_dir=cache_dir,
            revision=revision_corpus,
        )
        qrels_lang = load_dataset(
            qrels_path,
            lang,
            split=split,
            cache_dir=cache_dir,
            revision=revision_qrels,
        )
        # don't pass on titles to make task harder
        corpus_lang_dict = {doc["_id"]: {"text": doc["text"]} for doc in corpus_lang}
        queries_lang_dict = {query["_id"]: query["text"] for query in queries_lang}
        # qrels_lang_dict = {qrel["query-id"]: {qrel["corpus-id"]: qrel["score"]} for qrel in qrels_lang}

        qrels_lang_dict = {}
        for qrel in qrels_lang:
            if qrel["score"] == 0.5:
                continue
            # score = 0 if qrel["score"] == 0.5 else qrel["score"]
            # score = int(score)
            score = int(qrel["score"])
            qrels_lang_dict[qrel["query-id"]] = {qrel["corpus-id"]: score}

        corpus[lang][split] = corpus_lang_dict
        queries[lang][split] = queries_lang_dict
        qrels[lang][split] = qrels_lang_dict

    return corpus, queries, qrels


class WikipediaRetrievalMultilingual(MultilingualTask, AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="WikipediaRetrievalMultilingual",
        description="The dataset is derived from Cohere's wikipedia-2023-11 dataset and contains synthetically generated queries.",
        reference="https://huggingface.co/datasets/ellamind/wikipedia-2023-11-retrieval-multilingual-queries",
        dataset={
            "path": "ellamind/wikipedia-2023-11-retrieval-multilingual-queries",
            "revision": "3b6ea595c94bac3448a2ad167ca2e06abd340d6e",  # avoid validation error
            "revision_corpus": "f20ac0c449c85358d3d5c72a95f92f1eddc98aa5",
            "revision_qrels": "ec88a7bb2da034d538e98e3122d2c98530ca1c8d",
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=_EVAL_LANGS,
        main_score="ndcg_at_10",
        date=("2023-11-01", "2024-05-15"),
        domains=["Encyclopaedic", "Written"],
        task_subtypes=["Question answering", "Article retrieval"],
        license="cc-by-sa-3.0",
        annotations_creators="LM-generated and reviewed",
        dialect=[],
        sample_creation="LM-generated and verified",
        bibtex_citation="",
        descriptive_stats={
            "n_samples": {
                "en": 1500,
                "de": 1500,
                "it": 1500,
                "pt": 1500,
                "nl": 1500,
                "cs": 1500,
                "ro": 1500,
                "bg": 1500,
                "sr": 1500,
                "fi": 1500,
                "da": 1500,
                "fa": 1500,
                "hi": 1500,
                "bn": 1500,
                "no": 1500,
                "sv": 1500,
            },
            "avg_character_length": {
                "test": {
                    "bg": {
                        "average_document_length": 374.376,
                        "average_query_length": 1.0,
                        "num_documents": 13500,
                        "num_queries": 1500,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "bn": {
                        "average_document_length": 394.05044444444445,
                        "average_query_length": 1.0,
                        "num_documents": 13500,
                        "num_queries": 1500,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "cs": {
                        "average_document_length": 369.9831111111111,
                        "average_query_length": 1.0,
                        "num_documents": 13500,
                        "num_queries": 1500,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "da": {
                        "average_document_length": 345.2597037037037,
                        "average_query_length": 1.0,
                        "num_documents": 13500,
                        "num_queries": 1500,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "de": {
                        "average_document_length": 398.4137777777778,
                        "average_query_length": 1.0,
                        "num_documents": 13500,
                        "num_queries": 1500,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "en": {
                        "average_document_length": 452.9871111111111,
                        "average_query_length": 1.0,
                        "num_documents": 13500,
                        "num_queries": 1500,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "fa": {
                        "average_document_length": 345.1568888888889,
                        "average_query_length": 1.0,
                        "num_documents": 13500,
                        "num_queries": 1500,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "fi": {
                        "average_document_length": 379.71237037037037,
                        "average_query_length": 1.0,
                        "num_documents": 13500,
                        "num_queries": 1500,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "hi": {
                        "average_document_length": 410.72540740740743,
                        "average_query_length": 1.0,
                        "num_documents": 13500,
                        "num_queries": 1500,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "it": {
                        "average_document_length": 393.73437037037036,
                        "average_query_length": 1.0,
                        "num_documents": 13500,
                        "num_queries": 1500,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "nl": {
                        "average_document_length": 375.6695555555556,
                        "average_query_length": 1.0,
                        "num_documents": 13500,
                        "num_queries": 1500,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "pt": {
                        "average_document_length": 398.27237037037037,
                        "average_query_length": 1.0,
                        "num_documents": 13500,
                        "num_queries": 1500,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "ro": {
                        "average_document_length": 348.3817037037037,
                        "average_query_length": 1.0,
                        "num_documents": 13500,
                        "num_queries": 1500,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "sr": {
                        "average_document_length": 384.3131851851852,
                        "average_query_length": 1.0,
                        "num_documents": 13500,
                        "num_queries": 1500,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "no": {
                        "average_document_length": 366.93733333333336,
                        "average_query_length": 1.0,
                        "num_documents": 13500,
                        "num_queries": 1500,
                        "average_relevant_docs_per_query": 1.0,
                    },
                    "sv": {
                        "average_document_length": 369.340962962963,
                        "average_query_length": 1.0,
                        "num_documents": 13500,
                        "num_queries": 1500,
                        "average_relevant_docs_per_query": 1.0,
                    },
                }
            },
        },
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return

        self.corpus, self.queries, self.relevant_docs = _load_data(
            path=self.metadata_dict["dataset"]["path"],
            langs=self.hf_subsets,
            split=self.metadata_dict["eval_splits"][0],
            cache_dir=kwargs.get("cache_dir", None),
            revision_queries=self.metadata_dict["dataset"]["revision"],
            revision_corpus=self.metadata_dict["dataset"]["revision_corpus"],
            revision_qrels=self.metadata_dict["dataset"]["revision_qrels"],
        )

        self.data_loaded = True



from __future__ import annotations

from mteb.abstasks.AbsTaskSTS import AbsTaskSTS
from mteb.abstasks.TaskMetadata import TaskMetadata


class JSICK(AbsTaskSTS):
    metadata = TaskMetadata(
        name="JSICK",
        dataset={
            "path": "sbintuitions/JMTEB",
            "name": "jsick",
            "revision": "e4af6c73182bebb41d94cb336846e5a452454ea7",
            "trust_remote_code": True,
        },
        description="JSICK is the Japanese NLI and STS dataset by manually translating the English dataset SICK (Marelli et al., 2014) into Japanese.",
        reference="https://github.com/sbintuitions/JMTEB",
        type="STS",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["jpn-Jpan"],
        main_score="cosine_spearman",
        date=("2000-01-01", "2012-12-31"),  # best guess
        domains=["Web", "Written"],
        task_subtypes=[],
        license="cc-by-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""
        @article{yanaka2022compositional,
            title={Compositional Evaluation on Japanese Textual Entailment and Similarity},
            author={Yanaka, Hitomi and Mineshima, Koji},
            journal={Transactions of the Association for Computational Linguistics},
            volume={10},
            pages={1266--1284},
            year={2022},
            publisher={MIT Press One Broadway, 12th Floor, Cambridge, Massachusetts 02142, USA~…}
        }
        """,
        descriptive_stats={
            "n_samples": {"test": 1986},
            "avg_character_length": {"test": 21.47},
        },
    )

    @property
    def metadata_dict(self) -> dict[str, str]:
        metadata_dict = super().metadata_dict
        metadata_dict["min_score"] = 1
        metadata_dict["max_score"] = 5
        return metadata_dict

    def dataset_transform(self) -> None:
        self.dataset = self.dataset.rename_column("label", "score")



from __future__ import annotations

import logging
from collections import Counter, defaultdict

from datasets import Dataset

from ..encoder_interface import Encoder, EncoderWithQueryCorpusEncode
from ..evaluation.evaluators import PairClassificationEvaluator
from ..load_results.mteb_results import ScoresDict
from .AbsTask import AbsTask, DescriptiveStatistics

logger = logging.getLogger(__name__)


class PairClassificationDescriptiveStatistics(DescriptiveStatistics):
    """Descriptive statistics for PairClassification

    Attributes:
        num_samples: number of samples in the dataset.
        avg_sentence1_len: Average length of sentence1
        avg_sentence2_len: Average length of sentence2
        unique_labels: Number of unique labels
        labels: dict of label frequencies
    """

    num_samples: int
    avg_sentence1_len: float
    avg_sentence2_len: float
    unique_labels: int
    labels: dict[str, dict[str, int]]


class AbsTaskPairClassification(AbsTask):
    """Abstract class for PairClassificationTasks
    The similarity is computed between pairs and the results are ranked. Average precision
    is computed to measure how well the methods can be used for pairwise pair classification.

    self.load_data() must generate a huggingface dataset with a split matching self.metadata_dict["eval_splits"], and assign it to self.dataset. It must contain the following columns:
        sentence1: list[str]
        sentence2: list[str]
        labels: list[int]
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def _add_main_score(self, scores: ScoresDict) -> None:
        scores["main_score"] = scores[self.metadata.main_score]

    def _evaluate_subset(
        self,
        model: Encoder | EncoderWithQueryCorpusEncode,
        dataset: Dataset,
        *,
        encode_kwargs: dict[str, str] = {},
        **kwargs,
    ) -> ScoresDict:
        data_split = dataset[0]
        logging.getLogger(
            "sentence_transformers.evaluation.PairClassificationEvaluator"
        ).setLevel(logging.WARN)
        evaluator = PairClassificationEvaluator(
            data_split["sentence1"],
            data_split["sentence2"],
            data_split["labels"],
            task_name=self.metadata.name,
            **kwargs,
        )
        scores = evaluator.compute_metrics(model, encode_kwargs=encode_kwargs)

        self._add_main_score(scores)
        return scores

    def _calculate_metrics_from_split(
        self, split: str, hf_subset: str | None = None, compute_overall: bool = False
    ) -> PairClassificationDescriptiveStatistics:
        if hf_subset:
            dataset = self.dataset[hf_subset][split]
            if isinstance(dataset, list):
                dataset = dataset[0]
        elif compute_overall:
            dataset = defaultdict(list)
            for hf_subset in self.metadata.eval_langs:
                cur_dataset = self.dataset[hf_subset][split]
                if isinstance(cur_dataset, list):
                    cur_dataset = cur_dataset[0]
                for key, value in cur_dataset.items():
                    dataset[key].extend(value[0] if len(value) == 1 else value)
        else:
            dataset = self.dataset[split]

        sentence1 = (
            dataset["sentence1"][0]
            if len(dataset["sentence1"]) == 1
            else dataset["sentence1"]
        )
        sentence2 = (
            dataset["sentence2"][0]
            if len(dataset["sentence2"]) == 1
            else dataset["sentence2"]
        )
        labels = (
            dataset["labels"][0] if len(dataset["labels"]) == 1 else dataset["labels"]
        )

        total_sentence1_len = sum([len(sentence) for sentence in sentence1])
        total_sentence2_len = sum([len(sentence) for sentence in sentence2])
        label_count = Counter(labels)
        return PairClassificationDescriptiveStatistics(
            num_samples=len(sentence1),
            avg_sentence1_len=total_sentence1_len / len(sentence1),
            avg_sentence2_len=total_sentence2_len / len(sentence2),
            unique_labels=len(set(labels)),
            labels={
                str(label): {"count": count} for label, count in label_count.items()
            },
        )



from __future__ import annotations

from functools import partial
from typing import Any

import numpy as np

from mteb.encoder_interface import Encoder
from mteb.model_meta import ModelMeta


class GoogleTextEmbeddingModel(Encoder):
    def __init__(self, model_name: str, sep: str = " ", **kwargs) -> None:
        self.model_name = model_name

    def _embed(
        self,
        texts: list[str],
        google_task_type: str | None = None,
        titles: list[str] | None = None,
        dimensionality: int | None = 768,
    ) -> list[list[float]]:
        """Embeds texts with a pre-trained, foundational model.
        From https://cloud.google.com/vertex-ai/generative-ai/docs/embeddings/get-text-embeddings#generative-ai-get-text-embedding-python_vertex_ai_sdk
        """
        try:
            from vertexai.language_models import TextEmbeddingInput, TextEmbeddingModel
        except ImportError:
            raise ImportError(
                "The `vertexai` package is required to run the google API, please install it using `pip install vertexai`"
            )
        model = TextEmbeddingModel.from_pretrained(self.model_name)
        if titles:
            # Allow title-only embeddings by replacing text with a space
            # Else Google throws google.api_core.exceptions.InvalidArgument: 400 The text content is empty.
            inputs = [
                TextEmbeddingInput(
                    text if text else " ", task_type=google_task_type, title=title
                )
                for text, title in zip(texts, titles)
            ]
        else:
            inputs = [
                TextEmbeddingInput(text, task_type=google_task_type) for text in texts
            ]
        kwargs = {"output_dimensionality": dimensionality} if dimensionality else {}
        try:
            embeddings = model.get_embeddings(inputs, **kwargs)
        # Except the very rare google.api_core.exceptions.InternalServerError
        except Exception as e:
            print("Retrying once after error:", e)
            embeddings = model.get_embeddings(inputs, **kwargs)
        return np.asarray([embedding.values for embedding in embeddings])

    def encode(
        self,
        sentences: list[str],
        prompt_name: str | None = None,
        google_task_type: str | None = None,  # Optional
        **kwargs: Any,
    ) -> np.ndarray:
        if prompt_name and google_task_type is None:
            task = mteb.get_task(prompt_name)
            task_type = task.metadata.type
            if task_type in ["Classification", "MultilabelClassification"]:
                google_task_type = "CLASSIFICATION"
            elif task_type == "Clustering":
                google_task_type = "CLUSTERING"
            elif task_type == "STS":
                google_task_type = "SIMILARITY"
        return self._embed(sentences, google_task_type=google_task_type)

    def encode_queries(self, queries: list[str], **kwargs: Any) -> np.ndarray:
        return self._embed(queries, google_task_type="RETRIEVAL_QUERY")

    def encode_corpus(self, corpus: list[dict[str, str]], **kwargs: Any) -> np.ndarray:
        if isinstance(corpus, dict):
            sentences, titles = [], []

            for i in range(len(corpus["text"])):  # type: ignore
                titles.append(corpus["title"][i])  # type: ignore
                sentences.append(corpus["text"][i])  # type: ignore
        else:
            sentences, titles = [], []
            for doc in corpus:
                titles.append(doc["title"])
                sentences.append(doc["text"])
        return self._embed(
            sentences, google_task_type="RETRIEVAL_DOCUMENT", titles=titles
        )


name = "text-embedding-004"
google_emb_004 = ModelMeta(
    loader=partial(GoogleTextEmbeddingModel, model_name=name),
    name=name,
    languages=["eng-Latn"],
    open_source=False,
    revision="1",  # revision is intended for implementation
    release_date=None,  # couldnt figure this out
    n_parameters=None,
    memory_usage=None,
    max_tokens=2048,
    embed_dim=768,
    license=None,
    similarity_fn_name="cosine",  # assumed
    framework=[],
)


if __name__ == "__main__":
    import mteb

    mdl = mteb.get_model(google_emb_004.name, google_emb_004.revision)
    emb = mdl.encode(["Hello, world!"])



from __future__ import annotations

import logging
from typing import Any

from sentence_transformers import CrossEncoder, SentenceTransformer

from mteb.encoder_interface import Encoder, EncoderWithQueryCorpusEncode
from mteb.model_meta import ModelMeta
from mteb.models import (
    bge_models,
    bm25,
    cohere_models,
    e5_instruct,
    e5_models,
    google_models,
    gritlm_models,
    gte_models,
    llm2vec_models,
    mxbai_models,
    nomic_models,
    openai_models,
    promptriever_models,
    repllama_models,
    ru_sentence_models,
    salesforce_models,
    sentence_transformers_models,
    voyage_models,
)

logger = logging.getLogger(__name__)


def get_model(
    model_name: str, revision: str | None = None, **kwargs: Any
) -> Encoder | EncoderWithQueryCorpusEncode:
    """A function to fetch a model object by name.

    Args:
        model_name: Name of the model to fetch
        revision: Revision of the model to fetch
        **kwargs: Additional keyword arguments to pass to the model loader

    Returns:
        A model object
    """
    meta = get_model_meta(model_name, revision)
    model = meta.load_model(**kwargs)

    # If revision not available in the modelmeta, try to extract it from sentence-transformers
    if meta.revision is None and isinstance(model, SentenceTransformer):
        _meta = model_meta_from_sentence_transformers(model)
        meta.revision = _meta.revision if _meta.revision else meta.revision

    model.mteb_model_meta = meta  # type: ignore
    return model


def get_model_meta(model_name: str, revision: str | None = None) -> ModelMeta:
    """A function to fetch a model metadata object by name.

    Args:
        model_name: Name of the model to fetch
        revision: Revision of the model to fetch

    Returns:
        A model metadata object
    """
    if model_name in models:
        if revision and (not models[model_name].revision == revision):
            raise ValueError(
                f"Model revision {revision} not found for model {model_name}. Expected {models[model_name].revision}."
            )
        return models[model_name]

    # assume it is a sentence-transformers model
    logger.info(
        "Model not found in model registry, assuming it is a sentence-transformers model."
    )
    logger.info(
        f"Attempting to extract metadata by loading the model ({model_name}) using sentence-transformers."
    )
    model = SentenceTransformer(model_name, revision=revision, trust_remote_code=True)
    meta = model_meta_from_sentence_transformers(model)

    meta.revision = revision
    meta.name = model_name
    return meta


def model_meta_from_sentence_transformers(
    model: CrossEncoder | SentenceTransformer,
) -> ModelMeta:
    if isinstance(model, SentenceTransformer):
        name = (
            model.model_card_data.model_name
            if model.model_card_data.model_name
            else model.model_card_data.base_model
        )
        languages = (
            [model.model_card_data.language]
            if isinstance(model.model_card_data.language, str)
            else model.model_card_data.language
        )
        meta = ModelMeta(
            name=name,
            revision=model.model_card_data.base_model_revision,
            release_date=None,
            languages=languages,
            framework=["Sentence Transformers"],
            similarity_fn_name=model.similarity_fn_name,
        )
    elif isinstance(model, CrossEncoder):
        meta = ModelMeta(
            name=model.config._name_or_path,
            revision=None,
            release_date=None,
            languages=None,
            framework=["Sentence Transformers"],
            similarity_fn_name=None,
        )
    else:
        logger.warning(
            "Failed to extract metadata from model. Upgrading to sentence-transformers v3.0.0 or above is recommended."
        )
        meta = ModelMeta(
            name=None,
            revision=None,
            languages=None,
            release_date=None,
        )
    return meta


model_modules = [
    bge_models,
    bm25,
    cohere_models,
    e5_instruct,
    e5_models,
    google_models,
    gritlm_models,
    gte_models,
    llm2vec_models,
    mxbai_models,
    nomic_models,
    openai_models,
    promptriever_models,
    repllama_models,
    ru_sentence_models,
    salesforce_models,
    sentence_transformers_models,
    voyage_models,
    google_models,
]
models = {}

for module in model_modules:
    for mdl in vars(module).values():
        if isinstance(mdl, ModelMeta):
            models[mdl.name] = mdl



from __future__ import annotations

import datasets

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class BSARDRetrieval(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="BSARDRetrieval",
        description="The Belgian Statutory Article Retrieval Dataset (BSARD) is a French native dataset for studying legal information retrieval. BSARD consists of more than 22,600 statutory articles from Belgian law and about 1,100 legal questions posed by Belgian citizens and labeled by experienced jurists with relevant articles from the corpus.",
        reference="https://huggingface.co/datasets/maastrichtlawtech/bsard",
        dataset={
            "path": "maastrichtlawtech/bsard",
            "revision": "5effa1b9b5fa3b0f9e12523e6e43e5f86a6e6d59",
            "trust_remote_code": True,
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["fra-Latn"],
        main_score="recall_at_100",
        date=("2021-05-01", "2021-08-26"),
        domains=["Legal", "Spoken"],
        task_subtypes=[],
        license="cc-by-nc-sa-4.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@inproceedings{louis2022statutory,
  title = {A Statutory Article Retrieval Dataset in French},
  author = {Louis, Antoine and Spanakis, Gerasimos},
  booktitle = {Proceedings of the 60th Annual Meeting of the Association for Computational Linguistics},
  month = may,
  year = {2022},
  address = {Dublin, Ireland},
  publisher = {Association for Computational Linguistics},
  url = {https://aclanthology.org/2022.acl-long.468/},
  doi = {10.18653/v1/2022.acl-long.468},
  pages = {6789–6803},
}""",
        descriptive_stats={
            "n_samples": {"test": 222},
            "avg_character_length": {
                "test": {
                    "average_document_length": 880.2900631820793,
                    "average_query_length": 144.77027027027026,
                    "num_documents": 22633,
                    "num_queries": 222,
                    "average_relevant_docs_per_query": 1.0,
                }
            },
        },
    )

    def load_data(self, **kwargs):
        if self.data_loaded:
            return
        # fetch both subsets of the dataset, only test split
        corpus_raw = datasets.load_dataset(
            name="corpus",
            split="corpus",
            **self.metadata_dict["dataset"],
        )
        queries_raw = datasets.load_dataset(
            name="questions",
            split=self.metadata.eval_splits[0],
            **self.metadata_dict["dataset"],
        )

        self.queries = {
            self.metadata.eval_splits[0]: {
                str(q["id"]): (q["question"] + " " + q["extra_description"]).strip()
                for q in queries_raw
            }
        }

        self.corpus = {
            self.metadata.eval_splits[0]: {
                str(d["id"]): {"text": d["article"]} for d in corpus_raw
            }
        }

        self.relevant_docs = {self.metadata.eval_splits[0]: {}}
        for q in queries_raw:
            for doc_id in q["article_ids"]:
                self.relevant_docs[self.metadata.eval_splits[0]][str(q["id"])] = {
                    str(doc_id): 1
                }

        self.data_loaded = True



from __future__ import annotations

from mteb.abstasks.AbsTaskClassification import AbsTaskClassification
from mteb.abstasks.TaskMetadata import TaskMetadata


class KlueTC(AbsTaskClassification):
    metadata = TaskMetadata(
        name="KLUE-TC",
        dataset={
            "path": "klue/klue",
            "name": "ynat",
            "revision": "349481ec73fff722f88e0453ca05c77a447d967c",
        },
        description="Topic classification dataset of human-annotated news headlines. Part of the Korean Language Understanding Evaluation (KLUE).",
        reference="https://arxiv.org/abs/2105.09680",
        type="Classification",
        category="s2s",
        modalities=["text"],
        eval_splits=["validation"],
        eval_langs=["kor-Hang"],
        main_score="accuracy",
        date=("2016-01-01", "2020-12-31"),  # from 2016 to 2020
        domains=["News", "Written"],
        task_subtypes=["Topic classification"],
        license="cc-by-sa-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@misc{park2021klue,
      title={KLUE: Korean Language Understanding Evaluation}, 
      author={Sungjoon Park and Jihyung Moon and Sungdong Kim and Won Ik Cho and Jiyoon Han and Jangwon Park and Chisung Song and Junseong Kim and Yongsook Song and Taehwan Oh and Joohong Lee and Juhyun Oh and Sungwon Lyu and Younghoon Jeong and Inkwon Lee and Sangwoo Seo and Dongjun Lee and Hyunwoo Kim and Myeonghwa Lee and Seongbo Jang and Seungwon Do and Sunkyoung Kim and Kyungtae Lim and Jongwon Lee and Kyumin Park and Jamin Shin and Seonghyun Kim and Lucy Park and Alice Oh and Jungwoo Ha and Kyunghyun Cho},
      year={2021},
      eprint={2105.09680},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}""",
        descriptive_stats={
            "n_samples": {"validation": 2048},
            "avg_character_length": {"validation": 27.079609091907326},
        },
    )

    def dataset_transform(self):
        def id2str(example):
            return {"label": label_feature.int2str(example["label_id"])}

        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["validation"]
        )

        label_feature = self.dataset[self.metadata.eval_splits[0]].features["label"]

        self.dataset = self.dataset.rename_columns(
            {"title": "text", "label": "label_id"}
        )
        self.dataset = self.dataset.map(id2str)



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskSTS import AbsTaskSTS


class KorSTS(AbsTaskSTS):
    metadata = TaskMetadata(
        name="KorSTS",
        dataset={
            "path": "dkoterwa/kor-sts",
            "revision": "016f35f9b961daaaa7a352e927084e3da662ac1f",
        },
        description="Benchmark dataset for STS in Korean. Created by machine translation and human post editing of the STS-B dataset.",
        reference="https://arxiv.org/abs/2004.03289",
        type="STS",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["kor-Hang"],
        main_score="cosine_spearman",
        date=("2012-01-01", "2017-01-01"),  # rough approximates
        domains=["News", "Web"],
        task_subtypes=None,
        license="cc-by-sa-4.0",
        annotations_creators=None,
        dialect=[],
        sample_creation="machine-translated and localized",
        bibtex_citation="""@article{ham2020kornli,
  title={KorNLI and KorSTS: New Benchmark Datasets for Korean Natural Language Understanding},
  author={Ham, Jiyeon and Choe, Yo Joong and Park, Kyubyong and Choi, Ilji and Soh, Hyungjoon},
  journal={arXiv preprint arXiv:2004.03289},
  year={2020}
}""",
        descriptive_stats={
            "n_samples": {"test": 1379},
            "avg_character_length": {"test": 29.279433139534884},
        },
    )

    @property
    def metadata_dict(self) -> dict[str, str]:
        metadata_dict = super().metadata_dict
        metadata_dict["min_score"] = 0
        metadata_dict["max_score"] = 5
        return metadata_dict



from __future__ import annotations

from mteb.abstasks.AbsTaskClusteringFast import AbsTaskClusteringFast
from mteb.abstasks.TaskMetadata import TaskMetadata


class LivedoorNewsClusteringv2(AbsTaskClusteringFast):
    max_document_to_embed = 1107
    max_fraction_of_documents_to_embed = None

    metadata = TaskMetadata(
        name="LivedoorNewsClustering.v2",
        description="Clustering of the news reports of a Japanese news site, Livedoor News by RONDHUIT Co, Ltd. in 2012. It contains over 7,000 news report texts across 9 categories (topics). Version 2 updated on LivedoorNewsClustering by removing pairs where one of entries contain an empty sentences.",
        reference="https://github.com/sbintuitions/JMTEB",
        dataset={
            "path": "sbintuitions/JMTEB",
            "name": "livedoor_news",
            "revision": "e4af6c73182bebb41d94cb336846e5a452454ea7",
            "trust_remote_code": True,
        },
        type="Clustering",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["jpn-Jpan"],
        main_score="v_measure",
        date=("2000-01-01", "2014-02-09"),
        domains=["News", "Written"],
        task_subtypes=["Topic classification"],
        license="cc-by-nd-2.1-jp",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="",
        descriptive_stats={
            "n_samples": {"test": 1106},
            "avg_character_length": {"test": 1082.61},
        },
    )

    def dataset_transform(self):
        self.dataset = self.dataset.rename_columns(
            {"text": "sentences", "label": "labels"}
        )

        for split in self.metadata.eval_splits:
            # remove empty sentences (there is only one per split)
            self.dataset[split] = self.dataset[split].filter(
                lambda x: len(x["sentences"]) > 0
            )


class LivedoorNewsClustering(AbsTaskClusteringFast):
    max_document_to_embed = 1107
    max_fraction_of_documents_to_embed = None
    superseded_by = "LivedoorNewsClustering.v2"

    metadata = TaskMetadata(
        name="LivedoorNewsClustering",
        description="Clustering of the news reports of a Japanese news site, Livedoor News by RONDHUIT Co, Ltd. in 2012. It contains over 7,000 news report texts across 9 categories (topics).",
        reference="https://github.com/sbintuitions/JMTEB",
        dataset={
            "path": "sbintuitions/JMTEB",
            "name": "livedoor_news",
            "revision": "e4af6c73182bebb41d94cb336846e5a452454ea7",
            "trust_remote_code": True,
        },
        type="Clustering",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["jpn-Jpan"],
        main_score="v_measure",
        date=("2000-01-01", "2014-02-09"),
        domains=["News", "Written"],
        task_subtypes=["Topic classification"],
        license="cc-by-nd-2.1-jp",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="",
        descriptive_stats={
            "n_samples": {"test": 1107},
            "avg_character_length": {"test": 1082.61},
        },
    )

    def dataset_transform(self):
        self.dataset = self.dataset.rename_columns(
            {"text": "sentences", "label": "labels"}
        )



from __future__ import annotations

from mteb.abstasks.AbsTaskClassification import AbsTaskClassification
from mteb.abstasks.MultilingualTask import MultilingualTask
from mteb.abstasks.TaskMetadata import TaskMetadata

_LANGUAGES = {
    "ara": ["ara-Arab"],
    "cmn": ["cmn-Hans"],
    "eng": ["eng-Latn"],
    "deu": ["deu-Latn"],
    "fra": ["fra-Latn"],
    "hin": ["hin-Deva"],
    "ita": ["ita-Latn"],
    "nld": ["nld-Latn"],
    "pol": ["pol-Latn"],
    "por": ["por-Latn"],
    "spa": ["spa-Latn"],
}


class MultiHateClassification(MultilingualTask, AbsTaskClassification):
    fast_loading = True
    metadata = TaskMetadata(
        name="MultiHateClassification",
        dataset={
            "path": "mteb/multi-hatecheck",
            "revision": "8f95949846bb9e33c6aaf730ccfdb8fe6bcfb7a9",
        },
        description="""Hate speech detection dataset with binary
                       (hateful vs non-hateful) labels. Includes 25+ distinct types of hate
                       and challenging non-hate, and 11 languages.
                     """,
        reference="https://aclanthology.org/2022.woah-1.15/",
        type="Classification",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=_LANGUAGES,
        main_score="accuracy",
        date=("2020-11-23", "2022-02-28"),
        domains=["Constructed", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-4.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="created",
        bibtex_citation="""
        @inproceedings{rottger-etal-2021-hatecheck,
            title = "{H}ate{C}heck: Functional Tests for Hate Speech Detection Models",
            author = {R{\"o}ttger, Paul  and
            Vidgen, Bertie  and
            Nguyen, Dong  and
            Waseem, Zeerak  and
            Margetts, Helen  and
            Pierrehumbert, Janet},
            editor = "Zong, Chengqing  and
            Xia, Fei  and
            Li, Wenjie  and
            Navigli, Roberto",
            booktitle = "Proceedings of the 59th Annual Meeting of the Association for Computational Linguistics and the 11th International Joint Conference on Natural Language Processing (Volume 1: Long Papers)",
            month = aug,
            year = "2021",
            address = "Online",
            publisher = "Association for Computational Linguistics",
            url = "https://aclanthology.org/2021.acl-long.4",
            doi = "10.18653/v1/2021.acl-long.4",
            pages = "41--58",
            abstract = "Detecting online hate is a difficult task that even state-of-the-art models struggle with. Typically, hate speech detection models are evaluated by measuring their performance on held-out test data using metrics such as accuracy and F1 score. However, this approach makes it difficult to identify specific model weak points. It also risks overestimating generalisable model performance due to increasingly well-evidenced systematic gaps and biases in hate speech datasets. To enable more targeted diagnostic insights, we introduce HateCheck, a suite of functional tests for hate speech detection models. We specify 29 model functionalities motivated by a review of previous research and a series of interviews with civil society stakeholders. We craft test cases for each functionality and validate their quality through a structured annotation process. To illustrate HateCheck{'}s utility, we test near-state-of-the-art transformer models as well as two popular commercial models, revealing critical model weaknesses.",
        }

        @inproceedings{rottger-etal-2022-multilingual,
            title = "Multilingual {H}ate{C}heck: Functional Tests for Multilingual Hate Speech Detection Models",
            author = {R{\"o}ttger, Paul  and
            Seelawi, Haitham  and
            Nozza, Debora  and
            Talat, Zeerak  and
            Vidgen, Bertie},
            editor = "Narang, Kanika  and
            Mostafazadeh Davani, Aida  and
            Mathias, Lambert  and
            Vidgen, Bertie  and
            Talat, Zeerak",
            booktitle = "Proceedings of the Sixth Workshop on Online Abuse and Harms (WOAH)",
            month = jul,
            year = "2022",
            address = "Seattle, Washington (Hybrid)",
            publisher = "Association for Computational Linguistics",
            url = "https://aclanthology.org/2022.woah-1.15",
            doi = "10.18653/v1/2022.woah-1.15",
            pages = "154--169",
            abstract = "Hate speech detection models are typically evaluated on held-out test sets. However, this risks painting an incomplete and potentially misleading picture of model performance because of increasingly well-documented systematic gaps and biases in hate speech datasets. To enable more targeted diagnostic insights, recent research has thus introduced functional tests for hate speech detection models. However, these tests currently only exist for English-language content, which means that they cannot support the development of more effective models in other languages spoken by billions across the world. To help address this issue, we introduce Multilingual HateCheck (MHC), a suite of functional tests for multilingual hate speech detection models. MHC covers 34 functionalities across ten languages, which is more languages than any other hate speech dataset. To illustrate MHC{'}s utility, we train and test a high-performing multilingual hate speech detection model, and reveal critical model weaknesses for monolingual and cross-lingual applications.",
        }
        """,
        descriptive_stats={
            "n_samples": {"test": 10000},
            "avg_character_length": {"test": 45.9},
        },
    )

    def dataset_transform(self):
        # for each language perform some transforms
        for lang in self.dataset.keys():
            _dataset = self.dataset[lang]
            _dataset = _dataset.rename_columns({"is_hateful": "label"})
            for label in ["label", "functionality"]:
                _dataset = _dataset.class_encode_column(label)
            _dataset = _dataset["test"].train_test_split(
                train_size=1000,
                test_size=1000,
                seed=self.seed,
                stratify_by_column="functionality",
            )  # balanced sampling across types of hate speech
            _dataset = _dataset.remove_columns(["functionality"])
            self.dataset[lang] = _dataset



from __future__ import annotations

from typing import Any

from datasets import Dataset

from mteb.encoder_interface import Encoder, EncoderWithQueryCorpusEncode
from mteb.load_results.mteb_results import ScoresDict

from ..evaluation.evaluators import RerankingEvaluator
from .AbsTask import AbsTask, DescriptiveStatistics


class RerankingDescriptiveStatistics(DescriptiveStatistics):
    """Descriptive statistics for Reranking

    Attributes:
        num_samples: number of samples in the dataset.
        num_positive: Number of positive examples
        num_negative: Number of negative examples
        avg_query_len: Average length of queries
        avg_positive_len: Average length of positive examples
        avg_negative_len: Average length of negative examples
    """

    num_samples: int
    num_positive: int
    num_negative: int
    avg_query_len: float
    avg_positive_len: float
    avg_negative_len: float


class AbsTaskReranking(AbsTask):
    """Abstract class for re-ranking experiments.

    self.load_data() must generate a huggingface dataset with a split matching self.metadata_dict["eval_splits"], and assign it to self.dataset. It must contain the following columns:
        query: str
        positive: list[str]
        negative: list[str]
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def _evaluate_subset(
        self,
        model: Encoder | EncoderWithQueryCorpusEncode,
        data_split: Dataset,
        *,
        encode_kwargs: dict[str, Any] = {},
        **kwargs: Any,
    ) -> ScoresDict:
        evaluator = RerankingEvaluator(
            data_split,
            task_name=self.metadata.name,
            encode_kwargs=encode_kwargs,
            **kwargs,
        )
        scores = evaluator(model)

        self._add_main_score(scores)
        return scores

    def _add_main_score(self, scores: ScoresDict) -> None:
        scores["main_score"] = scores[self.metadata.main_score]

    def _calculate_metrics_from_split(
        self, split: str, hf_subset: str | None = None, compute_overall: bool = False
    ) -> RerankingDescriptiveStatistics:
        if hf_subset:
            query = self.dataset[hf_subset][split]["query"]
            positive = self.dataset[hf_subset][split]["positive"]
            negative = self.dataset[hf_subset][split]["negative"]
        elif compute_overall:
            query = []
            positive = []
            negative = []
            for hf_subset in self.metadata.eval_langs:
                query.extend(self.dataset[hf_subset][split]["query"])
                positive.extend(self.dataset[hf_subset][split]["positive"])
                negative.extend(self.dataset[hf_subset][split]["negative"])
        else:
            query = self.dataset[split]["query"]
            positive = self.dataset[split]["positive"]
            negative = self.dataset[split]["negative"]

        total_len_query = sum([len(q) for q in query])
        total_len_positive = sum([len(p) for p in positive])
        total_len_negative = sum([len(n) for n in negative])
        return RerankingDescriptiveStatistics(
            num_samples=len(query),
            num_positive=len(positive),
            num_negative=len(negative),
            avg_query_len=total_len_query / len(query),
            avg_positive_len=total_len_positive / len(positive),
            avg_negative_len=total_len_negative / len(negative),
        )



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class LegalBenchCorporateLobbying(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="LegalBenchCorporateLobbying",
        description="The dataset includes bill titles and bill summaries related to corporate lobbying.",
        reference="https://huggingface.co/datasets/nguha/legalbench/viewer/corporate_lobbying",
        dataset={
            "path": "mteb/legalbench_corporate_lobbying",
            "revision": "f69691c650464e62546d7f2a4536f8f87c891e38",
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=None,
        domains=["Legal", "Written"],
        task_subtypes=["Article retrieval"],
        license="cc-by-4.0",
        annotations_creators="derived",
        dialect=None,
        sample_creation="found",
        bibtex_citation="""@misc{guha2023legalbench,
      title={LegalBench: A Collaboratively Built Benchmark for Measuring Legal Reasoning in Large Language Models}, 
      author={Neel Guha and Julian Nyarko and Daniel E. Ho and Christopher Ré and Adam Chilton and Aditya Narayana and Alex Chohlas-Wood and Austin Peters and Brandon Waldon and Daniel N. Rockmore and Diego Zambrano and Dmitry Talisman and Enam Hoque and Faiz Surani and Frank Fagan and Galit Sarfaty and Gregory M. Dickinson and Haggai Porat and Jason Hegland and Jessica Wu and Joe Nudell and Joel Niklaus and John Nay and Jonathan H. Choi and Kevin Tobia and Margaret Hagan and Megan Ma and Michael Livermore and Nikon Rasumov-Rahe and Nils Holzenberger and Noam Kolt and Peter Henderson and Sean Rehaag and Sharad Goel and Shang Gao and Spencer Williams and Sunny Gandhi and Tom Zur and Varun Iyer and Zehua Li},
      year={2023},
      eprint={2308.11462},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}
@article{koreeda2021contractnli,
  title={ContractNLI: A dataset for document-level natural language inference for contracts},
  author={Koreeda, Yuta and Manning, Christopher D},
  journal={arXiv preprint arXiv:2110.01799},
  year={2021}
}
@article{hendrycks2021cuad,
  title={Cuad: An expert-annotated nlp dataset for legal contract review},
  author={Hendrycks, Dan and Burns, Collin and Chen, Anya and Ball, Spencer},
  journal={arXiv preprint arXiv:2103.06268},
  year={2021}
}
@article{wang2023maud,
  title={MAUD: An Expert-Annotated Legal NLP Dataset for Merger Agreement Understanding},
  author={Wang, Steven H and Scardigli, Antoine and Tang, Leonard and Chen, Wei and Levkin, Dimitry and Chen, Anya and Ball, Spencer and Woodside, Thomas and Zhang, Oliver and Hendrycks, Dan},
  journal={arXiv preprint arXiv:2301.00876},
  year={2023}
}
@inproceedings{wilson2016creation,
  title={The creation and analysis of a website privacy policy corpus},
  author={Wilson, Shomir and Schaub, Florian and Dara, Aswarth Abhilash and Liu, Frederick and Cherivirala, Sushain and Leon, Pedro Giovanni and Andersen, Mads Schaarup and Zimmeck, Sebastian and Sathyendra, Kanthashree Mysore and Russell, N Cameron and others},
  booktitle={Proceedings of the 54th Annual Meeting of the Association for Computational Linguistics (Volume 1: Long Papers)},
  pages={1330--1340},
  year={2016}
}
@inproceedings{zheng2021does,
  title={When does pretraining help? assessing self-supervised learning for law and the casehold dataset of 53,000+ legal holdings},
  author={Zheng, Lucia and Guha, Neel and Anderson, Brandon R and Henderson, Peter and Ho, Daniel E},
  booktitle={Proceedings of the eighteenth international conference on artificial intelligence and law},
  pages={159--168},
  year={2021}
}
@article{zimmeck2019maps,
  title={Maps: Scaling privacy compliance analysis to a million apps},
  author={Zimmeck, Sebastian and Story, Peter and Smullen, Daniel and Ravichander, Abhilasha and Wang, Ziqi and Reidenberg, Joel R and Russell, N Cameron and Sadeh, Norman},
  journal={Proc. Priv. Enhancing Tech.},
  volume={2019},
  pages={66},
  year={2019}
}
@article{ravichander2019question,
  title={Question answering for privacy policies: Combining computational and legal perspectives},
  author={Ravichander, Abhilasha and Black, Alan W and Wilson, Shomir and Norton, Thomas and Sadeh, Norman},
  journal={arXiv preprint arXiv:1911.00841},
  year={2019}
}
@article{holzenberger2021factoring,
  title={Factoring statutory reasoning as language understanding challenges},
  author={Holzenberger, Nils and Van Durme, Benjamin},
  journal={arXiv preprint arXiv:2105.07903},
  year={2021}
}
@article{lippi2019claudette,
  title={CLAUDETTE: an automated detector of potentially unfair clauses in online terms of service},
  author={Lippi, Marco and Pa{\l}ka, Przemys{\l}aw and Contissa, Giuseppe and Lagioia, Francesca and Micklitz, Hans-Wolfgang and Sartor, Giovanni and Torroni, Paolo},
  journal={Artificial Intelligence and Law},
  volume={27},
  pages={117--139},
  year={2019},
  publisher={Springer}
}
""",
        descriptive_stats={
            "n_samples": None,
            "avg_character_length": {
                "test": {
                    "average_document_length": 1157.2225705329154,
                    "average_query_length": 177.87941176470588,
                    "num_documents": 319,
                    "num_queries": 340,
                    "average_relevant_docs_per_query": 1.0,
                }
            },
        },
    )



from __future__ import annotations

from mteb.abstasks.AbsTaskClassification import AbsTaskClassification
from mteb.abstasks.TaskMetadata import TaskMetadata


class GermanPoliticiansTwitterSentimentClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="GermanPoliticiansTwitterSentimentClassification",
        description="GermanPoliticiansTwitterSentiment is a dataset of German tweets categorized with their sentiment (3 classes).",
        reference="https://aclanthology.org/2022.konvens-1.9",
        dataset={
            "path": "Alienmaster/german_politicians_twitter_sentiment",
            "revision": "65343b17f5a76227ab2e15b9424dfab6466ffcb1",
        },
        type="Classification",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["deu-Latn"],
        main_score="accuracy",
        date=("2021-01-01", "2021-12-31"),
        domains=["Social", "Government", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="not specified",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""
            @inproceedings{schmidt-etal-2022-sentiment,
                title = "Sentiment Analysis on {T}witter for the Major {G}erman Parties during the 2021 {G}erman Federal Election",
                author = "Schmidt, Thomas  and
                Fehle, Jakob  and
                Weissenbacher, Maximilian  and
                Richter, Jonathan  and
                Gottschalk, Philipp  and
                Wolff, Christian",
                editor = "Schaefer, Robin  and
                Bai, Xiaoyu  and
                Stede, Manfred  and
                Zesch, Torsten",
                booktitle = "Proceedings of the 18th Conference on Natural Language Processing (KONVENS 2022)",
                month = "12--15 " # sep,
                year = "2022",
                address = "Potsdam, Germany",
                publisher = "KONVENS 2022 Organizers",
                url = "https://aclanthology.org/2022.konvens-1.9",
                pages = "74--87",
            }
        """,
        descriptive_stats={
            "n_samples": {"test": 357},
            "avg_character_length": {"test": 302.48},
        },
    )

    def dataset_transform(self):
        self.dataset = self.dataset.rename_column("majority_sentiment", "label")



from __future__ import annotations

from mteb.abstasks.TaskMetadata import TaskMetadata

from ....abstasks.AbsTaskRetrieval import AbsTaskRetrieval


class SCIDOCSPL(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="SCIDOCS-PL",
        description="SciDocs, a new evaluation benchmark consisting of seven document-level tasks ranging from citation prediction, to document classification and recommendation.",
        reference="https://allenai.org/data/scidocs",
        dataset={
            "path": "clarin-knext/scidocs-pl",
            "revision": "45452b03f05560207ef19149545f168e596c9337",
            "trust_remote_code": True,
        },
        type="Retrieval",
        category="s2p",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        date=None,
        domains=None,
        task_subtypes=None,
        license=None,
        annotations_creators=None,
        dialect=None,
        sample_creation=None,
        bibtex_citation="""@misc{wojtasik2024beirpl,
      title={BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language}, 
      author={Konrad Wojtasik and Vadim Shishkin and Kacper Wołowiec and Arkadiusz Janz and Maciej Piasecki},
      year={2024},
      eprint={2305.19840},
      archivePrefix={arXiv},
      primaryClass={cs.IR}
}""",
        descriptive_stats={
            "n_samples": None,
            "avg_character_length": {
                "test": {
                    "average_document_length": 1270.0791986592353,
                    "average_query_length": 80.671,
                    "num_documents": 25657,
                    "num_queries": 1000,
                    "average_relevant_docs_per_query": 4.928,
                }
            },
        },
    )



from __future__ import annotations

from mteb.abstasks.AbsTaskClassification import AbsTaskClassification

# type: ignore
from mteb.abstasks.TaskMetadata import TaskMetadata  # type: ignore


class KorFin(AbsTaskClassification):
    metadata = TaskMetadata(
        name="KorFin",
        dataset={
            "path": "amphora/korfin-asc",
            "revision": "07cc4a29341ef26e8614ae1139847f4d4888727d",
        },
        description="The KorFin-ASC is an extension of KorFin-ABSA, which is a financial sentiment analysis dataset including 8818 samples with (aspect, polarity) pairs annotated. The samples were collected from KLUE-TC and analyst reports from Naver Finance.",
        reference="https://huggingface.co/datasets/amphora/korfin-asc",
        type="Classification",
        category="s2s",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["kor-Hang"],
        main_score="accuracy",
        date=(
            "2022-01-01",
            "2022-12-31",
        ),  # Assumed date based on the citations in the paper
        domains=["News", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="cc-by-sa-4.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=""""
        @article{son2023removing,
        title={Removing Non-Stationary Knowledge From Pre-Trained Language Models for Entity-Level Sentiment Classification in Finance},
        author={Son, Guijin and Lee, Hanwool and Kang, Nahyeon and Hahm, Moonjeong},
        journal={arXiv preprint arXiv:2301.03136},
        year={2023}
        }
        """,
        descriptive_stats={
            "n_samples": {"test": 2048},
            "avg_character_length": {"test": 75.28},
        },
    )

    def dataset_transform(self):
        self.dataset = self.dataset.rename_columns(
            {"SRC": "text", "SENTIMENT": "label"}
        ).remove_columns(["SID", "TYPE", "ASPECT"])
        self.dataset = self.stratified_subsampling(
            self.dataset, seed=self.seed, splits=["test"]
        )
